"""
sleep
-----

This task allows to set the bolt process to sleep for the specified duration.
If no duration is specified, the process will sleep forever. This task can be
used to wait for a subprocess to start in cases where the subprocess might take
some time to initialize. You can also use it to wait forever in case that an
independent subprocess needs to be kept running in the background./

..  code-block:: python

    config = {
        'sleep': {
            'duration': 10      # Waits 10 seconds.
        }
    }
"""
import time

import bolt.api as api
import bolt.utils as btutils


class SleepTask(api.Task):
    def _configure(self):
        self.duration = self._optional("duration", btutils.FOREVER)

    def _execute(self):
        if self.duration == btutils.FOREVER:
            self._sleep_forever()
        else:
            self._sleep(self.duration)

    def _sleep(self, seconds):
        time.sleep(seconds)

    def _sleep_forever(self):
        while True:
            pass


def register_tasks(registry):
    registry.register_task("sleep", SleepTask())



"""
setup
-----

The ``setup`` task provides an automation hook to execute ``setup.py``
commands and options inside of Bolt. The task, in its simplest form,
assumes a default ``setup.py`` in the current working directory and
uses a ``build`` command as a default if no configuration is provided.

The task configuration allows spcifying a setup script, which by
default will be set to ``setup.py`` if no script is specified, a valid
command, and it command arguments. The following example shows how to
configure the task. ::

    config = {
        'setup':{
            'script': 'special_setup.py',
            'command': 'install',
            'options': {
                'verbose': True,
                'dry-run': True
            }
        }
    }
"""
import distutils.core as dcore

import bolt.api as api
import bolt.utils as utilities

DEFAULT_ARGUMENTS = ["build"]
DEFAULT_SETUP_SCRIPT = "setup.py"


class ExecuteSetupTask(api.Task):
    def _configure(self):
        self.setup_script = self._optional("script")
        if self.setup_script:
            self.config["script"] = False
        else:
            self.setup_script = DEFAULT_SETUP_SCRIPT
        generator = _SetupArgumentGenerator()
        self.args = generator.generate_from(self.config)

    def _execute(self):
        result = self._execute_setup()
        if not result.dist_files:
            raise BuildSetupError()

    def _execute_setup(self):
        return dcore.run_setup(self.setup_script, self.args)


def register_tasks(registry):
    registry.register_task("setup", ExecuteSetupTask())


class _SetupArgumentGenerator(utilities.CommonCommandAndArgumentsGenerator):
    def __init__(self):
        return super(_SetupArgumentGenerator, self).__init__(DEFAULT_ARGUMENTS)


class BuildSetupError(api.TaskFailedError):
    def __repr__(self):
        return "BuildSetupError({code})".format(code=self.code)



"""
This module defines exception classes used in the implementation of Bolt to
report errors or as base classes to define other error conditions.
"""
from bolt.api import BoltError


class InvalidConfigurationError(BoltError):
    """
    Base class for all task configuration errors.
    """

    def __repr__(self):
        return "InvalidConfigurationError()"


class ConfigurationParameterError(InvalidConfigurationError):
    """
    Base class for configuration parameters errors.
    """

    def __init__(self, param_name):
        self.parameter = param_name

    def __repr__(self):
        return "ConfigurationParameterError({pn})".format(pn=repr(self.parameter))


class RequiredParameterMissingError(ConfigurationParameterError):
    """
    Indicates a requiered configuration parameter is missing from the task
    configuration
    """

    def __repr__(self):
        return "RequiredParameterMissingError({pn})".format(pn=repr(self.parameter))


class ConfigurationValueError(ConfigurationParameterError):
    """
    Indicates the value for a configuration parameter is invalid.
    """

    def __init__(self, param_name, value):
        super(ConfigurationValueError, self).__init__(param_name)
        self.value = value

    def __repr__(self):
        fmt = "ConfigurationValueError({pn}, {v})"
        return fmt.format(pn=repr(self.parameter), v=repr(self.value))


class TaskError(BoltError):
    """
    Base class for exceptions raised during task execution. Developers should
    raise this exception or a derived type to indicate an error during the
    execution of the task they are implementing.
    """

    def __repr__(self):
        return "TaskError()"



import unittest

import bolt._btregistry as registry


class TestTaskRegistry(unittest.TestCase):
    def setUp(self):
        self.subject = registry.TaskRegistry()

    def test_can_register_callable_as_task(self):
        task_name = "test"
        self.subject.register_task(task_name, self.test_callable)
        actual = self.subject.get(task_name)
        self.assertEqual(actual, self.test_callable)

    def test_can_register_list_of_task_names(self):
        task_name = "list"
        task_list = ["task1", "task2", "task3"]
        self.subject.register_task(task_name, task_list)
        actual_list = self.subject.get(task_name)
        self.assertSequenceEqual(task_list, actual_list)

    def test_fails_if_task_is_not_callable_or_list(self):
        with self.assertRaises(registry.InvalidTaskTypeError):
            self.subject.register_task("invalid", 1)

    def test_raises_if_list_contains_anything_other_than_strings(self):
        with self.assertRaises(registry.InvalidTaskTypeError):
            self.subject.register_task("invalid", ["foo", "bar", 3])

    def test_allows_specifying_subtask_returing_correct_callable(self):
        task_name = "test"
        subtask_name = "test.sub"
        self.subject.register_task(task_name, self.test_callable)
        actual = self.subject.get(subtask_name)
        self.assertEqual(actual, self.test_callable)

    def test_callable(self):
        pass


if __name__ == "__main__":
    unittest.main()



"""
.. _task-delete-files:

delete-files
------------

This task deletes files matching a specified ``pattern`` found in a specified
``sourcedir``. A ``recursive`` flag can be specified to search also in
sub-directories of ``sourcedir``.

The ``pattern`` specified follows the matching rules of the |python|_ standard
library ``glob.glob()`` function.

The following example configures the ``delete-file`` task to delete all the
files in a ``tmp`` directory located at the project root, and all its
sub-directories::

    config = {
        'delete-files': {
            'sourcedir': './tmp',
            'pattern': '*.*',
            'recursive': True
        }
    }

The ``sourcedir`` configuration option indicates the directory to search for
file matches. This option is required.

The ``pattern`` option specifies the matching pattern to find files. This
option is required.

The ``recursive`` option indicates if sub-directories should be searched for
matches. This option is optional and has a value of ``False`` by default.


.. _task-delete-pyc:

delete-pyc
----------

Searches for ``.pyc`` in the specified directory and deletes them. The task
allows to recursively search sub-directories for ``.pyc`` files.

The following example shows how to configure the task to recursively delete
files from a source directory and its sub-directories::

    config = {
        'delete-pyc': {
            'sourcedir': './source',
            'recursive': True
        }
    }

The ``sourcedir`` option specifies the directory to search for ``.pyc`` files.
This option is required.

The ``recursive`` option indicates if sub-directories should be searched for
matches. This option is optional and has a value of ``False`` by default.
"""
import logging

import bolt.api as api
import bolt.utils as utilities


class DeleteFilesTask(api.Task):
    def _configure(self):
        self.sourcedir = self._require("sourcedir")
        self.pattern = self._require("pattern")
        self.recursive = self._optional("recursive")

    def _execute(self):
        logging.info(
            "Deleting {pat} from {srcdir}.".format(
                pat=self.pattern, srcdir=self.sourcedir
            )
        )
        finder = utilities.FileFinder(self.sourcedir, self.pattern, self.recursive)
        matches = finder.find()
        utilities.delete_files_in(matches)


class DeletePycTask(DeleteFilesTask):
    def _configure(self):
        self.config["pattern"] = "*.pyc"
        logging.debug("Delete pattern set to " + self.config["pattern"])
        super(DeletePycTask, self)._configure()


def register_tasks(registry):
    registry.register_task("delete-files", DeleteFilesTask())
    registry.register_task("delete-pyc", DeletePycTask())



import logging
import os.path
import unittest

import bolt._btapp as btapp
import bolt.api as api
import bolt._btregistry as btregistry


class TestBoltApplication(unittest.TestCase):
    def setUp(self):
        self.options = OptionsDummy()
        self.registry = btregistry.TaskRegistry()
        self.config_manager = ConfigManagerDummy()
        self.application = btapp.BoltApplication(
            self.options, self.registry, self.config_manager
        )
        self.default_application = BoltApplicationSpy()
        return super(TestBoltApplication, self).setUp()

    def test_uses_specified_options(self):
        self.assertIs(self.application.options, self.options)

    def test_uses_default_options_if_none_specified(self):
        self.assertIs(
            self.default_application.options, self.default_application.default_options
        )

    def test_uses_specified_registry(self):
        self.assertIs(self.application.registry, self.registry)

    def test_uses_default_registry_if_none_specified(self):
        self.assertIs(
            self.default_application.registry, self.default_application.default_registry
        )

    def test_uses_specified_configuration_manager(self):
        self.assertIs(self.application.config_manager, self.config_manager)

    def test_delays_initialization_of_configuration_manager_if_not_specified(self):
        self.assertIsNone(self.default_application.config_manager)

    def test_runs_task_specified(self):
        self.setup_tasks()
        self.application.run_task("successful_task")
        self.assertTrue(self.successful_task_executed)

    def test_continues_on_error_if_specified(self):
        self.setup_tasks()
        self.application.options.continue_on_error = True
        self.application.run_task("continue")
        self.assertTrue(self.successful_task_executed)

    def test_can_override_continue_on_error(self):
        self.setup_tasks()
        self.application.run_task("continue", True)
        self.assertTrue(self.successful_task_executed)

    def test_raises_system_exit_if_no_continue_on_error_set(self):
        with self.assertRaises(api.TaskFailedError):
            self.setup_tasks()
            self.application.run_task("continue")
            self.assertFalse(self.successful_task_executed)

    def test_tears_down_tasks_at_the_end(self):
        self.setup_tasks()
        self.application.run_task("tearable")
        self.assertTrue(self.tearable_task.tear_down_invoked)

    def test_tears_down_even_if_exception(self):
        try:
            self.setup_tasks()
            self.application.run_task("insure_teardown")
        except api.TaskFailedError:
            self.assertTrue(self.tearable_task.tear_down_invoked)

    def test_executes_specified_task(self):
        self.setup_tasks()
        self.application.options.task = "successful_task"
        self.application.run()
        self.assertTrue(self.successful_task_executed)

    def test_standard_tasks_are_registered(self):
        self.assert_registered("conttest")
        self.assert_registered("coverage")
        self.assert_registered("delete-files")
        self.assert_registered("delete-pyc")
        self.assert_registered("mkdir")
        self.assert_registered("nose")
        self.assert_registered("pip")
        self.assert_registered("set-vars")
        self.assert_registered("setup")
        self.assert_registered("shell")
        self.assert_registered("sleep")

    def setup_tasks(self):
        self.tearable_task = TearableTask()
        self.application.registry.register_task("tearable", self.tearable_task)
        self.application.registry.register_task("successful_task", self.successful_task)
        self.application.registry.register_task("failed_task", self.failed_task)
        self.application.registry.register_task(
            "continue", ["failed_task", "successful_task"]
        )
        self.application.registry.register_task(
            "insure_teardown", ["tearable", "failed_task"]
        )

    def assert_registered(self, task_name):
        task = self.application.registry.get(task_name)
        self.assertIsNotNone(task)

    def successful_task(self, **kwargs):
        self.successful_task_executed = True
        return 0

    def failed_task(self, **kwargs):
        raise api.TaskFailedError()


class OptionsDummy:
    def __init__(self):
        self.log_level = logging.CRITICAL
        self.log_file = None
        self.continue_on_error = False


class ConfigManagerDummy:
    def get(self, task):
        return {}


class BoltApplicationSpy(btapp.BoltApplication):
    def __init__(self):
        self.default_options = OptionsDummy()
        self.default_registry = None
        self.default_config_manager = ConfigManagerDummy()
        return super(BoltApplicationSpy, self).__init__()

    def _get_default_options(self):
        return self.default_options

    def _get_default_registry(self):
        registry = super(BoltApplicationSpy, self)._get_default_registry()
        self.default_registry = registry
        return self.default_registry

    def _get_default_config_manager(self):
        return self.default_config_manager

    def _get_file_logger_handler(self, filename):
        return logging.StreamHandler()


class TearableTask:
    def __call__(self, **kwargs):
        pass

    def tear_down(self):
        self.tear_down_invoked = True


class LoggerDummy:
    def __init__(self):
        self.handler_count = 0

    def setLevel(self, level):
        self.level = level

    def addHandler(self, handler):
        self.handler_count += 1


class TestBoltFile(unittest.TestCase):
    def test_loads_specified_file(self):
        bfile = os.path.abspath("a_bolt_file.py")
        boltfile = BoltFileSpy(bfile)
        self.assertEqual(boltfile.filename, bfile)

    def test_returns_config_from_module(self):
        boltfile = BoltFileSpy("a_bolt_file.py")
        self.assertEqual(boltfile.config, boltfile.expected_module.config)


class BoltFileSpy(btapp.BoltFile):
    def _load(self):
        self.expected_module = BoltModuleDouble()
        return self.expected_module


class BoltModuleDouble:
    @property
    def config(self):
        return {}


if __name__ == "__main__":
    unittest.main()



"""

"""
project = "bolt-ta"
version = "0.3"
release = "0.3.5"
description = "A task runner written in Python"
copyright = "2016 Abantos"
author = "Isaac Rodriguez"
author_email = "oss.abantos@outlook.com"
url = "https://github.com/abantos/bolt"
license = "MIT"
keywords = "automation task tool development"
# https://pypi.org/pypi?%3Aaction=list_classifiers
classifiers = [
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Topic :: Software Development :: Build Tools",
    "Topic :: Utilities",
]



"""
Exposes classes related to the supported options for Bolt.
"""
import argparse
import logging


class Commands:
    """
    Defines the supported commands.
    """

    EXECUTE = "execute"
    HELP = "help"
    VERSION = "version"


class Default:
    """
    Defines default values for options.
    """

    COMMAND = Commands.EXECUTE
    TASK = "default"
    BOLTFILE = "boltfile.py"
    LOG_LEVEL = logging.INFO
    LOG_FILE = None
    CONTINUE_ON_ERROR = False


class OptionSwitch:
    """
    Defines values for the command line switches.
    """

    TASK = "task"
    BOLTFILE_SHORT = "-b"
    BOLTFILE_LONG = "--bolt-file"
    LOG_LEVEL_SHORT = "-v"
    LOG_LEVEL_LONG = "--log-level"
    LOG_FILE_SHORT = "-o"
    LOG_FILE_LONG = "--log-file"
    CONTINUE_ON_ERROR_LONG = "--continue-on-error"
    VERSION_LONG = "--version"


class Options:
    """
    Exposes application options that are initialized from the command line or
    by specifying a parameters list.
    """

    def __init__(self, opt_list=None):
        self._command = Commands.EXECUTE
        self._initialize_options(opt_list)

    @property
    def command(self):
        """
        Returns the command to be executed.
        """
        return self._command

    @property
    def task(self):
        """
        Returns the task to be executed or the default task if none specified.
        """
        return self._options.task

    @property
    def bolt_file(self):
        """
        Returns the bolt file to use or the default bolt file if none specified.
        """
        return self._options.bolt_file

    @property
    def log_level(self):
        """
        Returns the log level to use or the default log level if none specified.
        """
        return self._options.log_level

    @property
    def log_file(self):
        """
        Returns the log file to use or None if no file is specified.
        """
        return self._options.log_file

    @property
    def continue_on_error(self):
        """
        Returns whether to continue executing tasks when errors are found. The
        default is false.
        """
        return self._options.continue_on_error

    def _initialize_options(self, opt_list=None):
        parser = self._build_parser()
        self._options = parser.parse_args(opt_list)
        self._marshall_options()

    def _build_parser(self):
        parser = argparse.ArgumentParser()
        parser.add_argument(
            OptionSwitch.TASK,
            nargs="?",
            default=Default.TASK,
            help='Task to be executed. It uses "default" if not specified',
        )
        parser.add_argument(
            OptionSwitch.BOLTFILE_SHORT,
            OptionSwitch.BOLTFILE_LONG,
            default=Default.BOLTFILE,
            help='Bolt file containing the configuration and task. The default is "boltfile.py"',
        )
        parser.add_argument(
            OptionSwitch.LOG_LEVEL_SHORT,
            OptionSwitch.LOG_LEVEL_LONG,
            default=Default.LOG_LEVEL,
            help="Specifies logging level.",
        )
        parser.add_argument(
            OptionSwitch.LOG_FILE_SHORT,
            OptionSwitch.LOG_FILE_LONG,
            default=None,
            help="Log file to write the output.",
        )
        parser.add_argument(
            OptionSwitch.CONTINUE_ON_ERROR_LONG,
            action="store_true",
            help="Continue executing even if errors are found",
        )
        parser.add_argument(
            OptionSwitch.VERSION_LONG,
            action="store_true",
            help="Shows the the application version",
        )
        return parser

    def _marshall_options(self):
        self._marshall_command()
        self._marshall_log_level()

    def _marshall_command(self):
        if self._options.version:
            self._command = Commands.VERSION

    def _marshall_log_level(self):
        if self._options.log_level == Default.LOG_LEVEL:
            return
        lc_level = self._options.log_level.lower()
        if lc_level in ["", "n", "notset"]:
            self._options.log_level = logging.NOTSET
        elif lc_level in ["debug", "d", "dbg"]:
            self._options.log_level = logging.DEBUG
        elif lc_level in ["info", "i", "information"]:
            self._options.log_level = logging.INFO
        elif lc_level in ["warning", "warn", "w"]:
            self._options.log_level = logging.WARNING
        elif lc_level in ["error", "err", "e"]:
            self._options.log_level = logging.ERROR
        elif lc_level in ["critical", "crit", "c"]:
            self._options.log_level = logging.CRITICAL



import unittest

import bolt.api as api


class TestTask(unittest.TestCase):
    def setUp(self):
        self.config = {"param": "value"}
        self.task = TaskSpy()

    def test_retrieves_config_attribute_on_call(self):
        self.invoke()
        self.assertIs(self.task.config, self.config)

    def test_delegates_configuration_to_derived_classes(self):
        self.invoke()
        self.assertTrue(self.task.configure_invoked)

    def test_delegate_execution_to_derived_classes(self):
        self.invoke()
        self.assertTrue(self.task.execute_invoked)

    def test_raises_if_required_parameter_is_missing_in_configuration(self):
        with self.assertRaises(api.RequiredConfigurationError):
            self.invoke({})

    def test_sets_optional_to_default_value_if_not_specified_in_config(self):
        self.invoke()
        self.assertEqual(self.task.other, 3)

    def test_sets_optional_to_none_if_not_specified_and_no_default_provided(self):
        self.invoke()
        self.assertIsNone(self.task.must_be_none)

    def invoke(self, config=None):
        config = config if config is not None else self.config
        self.task(config=config)


class TaskSpy(api.Task):
    def __init__(self):
        self.configure_invoked = False
        self.execute_invoked = False

    def _configure(self):
        self.configure_invoked = True
        self.param = self._require("param")
        self.other = self._optional("other", 3)
        self.must_be_none = self._optional("no-default")

    def _execute(self):
        self.execute_invoked = True


class TestBoltError(unittest.TestCase):
    def setUp(self):
        self.exception = api.BoltError()

    def test_code_is_1_by_default(self):
        self.assertEqual(self.exception.code, 1)

    def test_code_can_be_specified(self):
        self.assertEqual(api.BoltError(2).code, 2)

    def test_provides_repr(self):
        self.assertEqual(repr(self.exception), "BoltError(1)")

    def test_string_representation_uses_repr_by_default(self):
        self.assertEqual(str(self.exception), repr(self.exception))


class TestTaskError(unittest.TestCase):
    def setUp(self):
        self.exception = api.TaskError()

    def test_code_is_999_by_default(self):
        self.assertEqual(self.exception.code, 999)

    def test_overrides_repr_to_show_generic_name(self):
        self.assertEqual(repr(self.exception), "TaskError(999)")


class TestRequiredConfigurationError(unittest.TestCase):
    def setUp(self):
        self.param = "param-name"
        self.exception = api.RequiredConfigurationError(self.param)

    def test_uses_code_2(self):
        self.assertEqual(self.exception.code, 2)

    def test_requires_parameter_name(self):
        self.assertEqual(self.exception.parameter, self.param)

    def test_shows_parameter_value_in_representation(self):
        self.assertEqual(repr(self.exception), "RequiredConfigurationError(param-name)")


class TestConfigurationValueError(unittest.TestCase):
    def setUp(self):
        self.param = "param"
        self.value = "value"
        self.exception = api.ConfigurationValueError(self.param, self.value)

    def test_uses_code_3(self):
        self.assertEqual(self.exception.code, 3)

    def test_requires_parameter_name(self):
        self.assertEqual(self.exception.parameter, "param")

    def test_requires_error_value(self):
        self.assertEqual(self.exception.value, self.value)

    def test_shows_parameter_and_value_in_representation(self):
        self.assertEqual(repr(self.exception), "ConfigurationValueError(param, value)")


class TestTaskFailedError(unittest.TestCase):
    def setUp(self):
        self.exception = api.TaskFailedError()

    def test_uses_code_4(self):
        self.assertEqual(self.exception.code, 4)

    def test_overrides_repr_to_show_generic_name(self):
        self.assertEqual(repr(self.exception), "TaskFailedError(4)")


if __name__ == "__main__":
    unittest.main()



"""
Classes to help processing configurations.
"""

TO_ENABLE_FLAG = True
TO_DISABLE_FLAG = False


def append_with_equal(arg_list, formatted_option, value):
    if value is TO_ENABLE_FLAG:
        arg_list.append(formatted_option)
    elif value is not TO_ENABLE_FLAG:
        formatted_argument = "{option}={value}".format(
            option=formatted_option, value=value
        )
        arg_list.append(formatted_argument)


def append_as_tokens(arg_list, formatted_option, value):
    arg_list.append(formatted_option)
    if value is not TO_ENABLE_FLAG:
        arg_list.append(value)


class ArgumentsGenerator:
    def __init__(self, default_args, append_callback=None):
        self.default_arguments = default_args
        self.append_callback = append_callback or append_as_tokens

    def generate_from(self, config):
        self.args = []
        if not config:
            return self.default_arguments
        self.config = config
        self._convert_config_to_arguments()
        return self.args

    def _convert_config_to_arguments(self):
        self.options = self.config.get("options")
        if self.options:
            {
                self._push_as_arguments(option, value)
                for option, value in self.options.items()
            }

    def _push_as_arguments(self, option, value):
        if value is not TO_DISABLE_FLAG:
            self._do_push(option, value)

    def _do_push(self, option, value):
        formatted_option = self._format_option(option)
        self.append_callback(self.args, formatted_option, value)

    def _format_option(self, option):
        if len(option) == 1:
            fmt_str = "-{option}"
        else:
            fmt_str = "--{option}"
        formatted_option = fmt_str.format(option=option)
        return formatted_option


class CommonCommandAndArgumentsGenerator(ArgumentsGenerator):
    def _convert_config_to_arguments(self):
        self.command = self.config.get("command")
        self.args.append(self.command)
        return super(
            CommonCommandAndArgumentsGenerator, self
        )._convert_config_to_arguments()



"""
This module contains classes that can be used for the implementation of
bolt tasks.
"""
import logging


class Task:
    """ """

    def __call__(self, **kwargs):
        self.config = kwargs.get("config")
        self._configure()
        self._execute()

    def _configure(self):
        """ """
        logging.warning("Derived classes should implement _configure()")

    def _execute(self):
        """ """
        logging.warning("Derived classes should implement _execute()")

    def _require(self, key):
        value = self.config.get(key)
        if not value:
            raise RequiredConfigurationError(key)
        return value

    def _optional(self, key, default=None):
        return self.config.get(key, default)


class BoltError(Exception):
    """
    Base class for all exceptions in Bolt. The class is provided for inheritance
    and it should not be used by itself.

    Errors for the Bolt application itself should be derived from this class.
    """

    def __init__(self, code=1):
        self.code = code

    def __repr__(self):
        return "BoltError({code})".format(code=self.code)

    def __str__(self):
        return repr(self)


class TaskError(BoltError):
    """
    Base class for exceptions raised by a bolt task implementation. This class
    is provided for inheritance and it should not be used by itself.

    Exceptions derived from this class should be implemented to indicate errors
    during the execution of a class. More specific exception base classes are
    provided for the distinct processes of executing a class.

    ..  seealso::
        :class:`RequiredConfigurationError`
        :class:`ConfigurationValueError`
        :class:`TaskFailedError`
    """

    def __init__(self, code=999):
        super(TaskError, self).__init__(code)

    def __repr__(self):
        return "TaskError({code})".format(code=self.code)


class RequiredConfigurationError(TaskError):
    """
    Exception raised by tasks when a required configuration parameter is missing.

    :param str parameter:
        Name of the parameter missing.
    """

    def __init__(self, parameter):
        super(RequiredConfigurationError, self).__init__(2)
        self.parameter = parameter

    def __repr__(self):
        return "RequiredConfigurationError({param})".format(param=self.parameter)


class ConfigurationValueError(TaskError):
    """
    Exception raised by tasks when the value of a configuration parameter is
    invalid.

    :param str parameter:
        Name of the parameter that has an invalid value.
    :param any value:
        Invalid value specified.
    """

    def __init__(self, parameter, value):
        super(ConfigurationValueError, self).__init__(3)
        self.parameter = parameter
        self.value = value

    def __repr__(self):
        return "ConfigurationValueError({param}, {value})".format(
            param=self.parameter, value=self.value
        )


class TaskFailedError(TaskError):
    """
    Exception to indicate that a task execution has failed. Tasks can choose
    to raise this exception directly or derive from a more specific exception
    to provide more information about the error.
    """

    def __init__(self, code=4):
        super(TaskFailedError, self).__init__(code)

    def __repr__(self):
        return "TaskFailedError({code})".format(code=self.code)



"""
.. _task-nose:

nose
----

Executes unit tests using nose and nosetests as the unit test runner. The task
allows to specify the directory where the tests are located through the ``directory``
parameter and supports all the arguments available in the installed version
of nosetests::

    config = {
        'nose': {
            'directory': 'test/unit',
            'options': {
                'xunit-file': 'output/unit_tests.xml'
                'with-coverage': True,
                'cover-erase': True,
                'cover-package': 'mypackage',
                'cover-html': True,
                'cover-html-dir': 'output/coverage',
            }
    }
"""
import logging
import os.path
import subprocess as sp

import bolt.api as api
import bolt.utils as utilities


DEFAULT_DIR = "./"
DEFAULT_ARGUMENTS = [DEFAULT_DIR]


class ExecuteNoseTask(api.Task):
    def _configure(self):
        generator = _NoseArgumentGenerator()
        self.args = generator.generate_from(self.config)
        logging.debug("Arguments: " + repr(self.args))

    def _execute(self):
        result = sp.call(self.args)
        if result != 0:
            raise NoseError(result)


def register_tasks(registry):
    registry.register_task("nose", ExecuteNoseTask())


class NoseError(api.TaskFailedError):
    def __init__(self, nose_code):
        super(NoseError, self).__init__(nose_code)

    def __repr__(self):
        return "NoseError({code})".format(code=self.code)


class _NoseArgumentGenerator(utilities.ArgumentsGenerator):
    def __init__(self):
        return super(_NoseArgumentGenerator, self).__init__(
            DEFAULT_ARGUMENTS, utilities.append_with_equal
        )

    def _convert_config_to_arguments(self):
        self.args.append("nosetests")
        super(_NoseArgumentGenerator, self)._convert_config_to_arguments()
        directory = self.config.get("directory") or DEFAULT_DIR
        directory = os.path.abspath(directory)
        self.args.append(directory)



"""
"""
import logging
import unittest

import bolt._btoptions as btoptions


class TestOptions(unittest.TestCase):
    def setUp(self):
        # If we don't initialize it from an empty options list, it parses
        # the arguments to nosetests and the test fail. The actual code using
        # it doesn't need to do it because it will parse the arguments to
        # bolt.
        self.default_options = btoptions.Options([])
        return super(TestOptions, self).setUp()

    def test_default_command_is_properly_initialized(self):
        self.assertEqual(self.default_options.command, btoptions.Default.COMMAND)

    def test_command_is_version_if_switch_specified(self):
        self.given(btoptions.OptionSwitch.VERSION_LONG)
        self.assertEqual(self.options.command, btoptions.Commands.VERSION)

    def test_default_task_is_properly_initialized(self):
        self.assertEqual(self.default_options.task, btoptions.Default.TASK)

    def test_returns_correct_task_if_specified(self):
        task = "a_task"
        self.given(task)
        self.assertEqual(self.options.task, task)

    def test_default_boltfile_is_properly_initialized(self):
        self.assertEqual(self.default_options.bolt_file, btoptions.Default.BOLTFILE)

    def test_sets_boltfile_with_long_switch(self):
        boltfile = "a_bolt_file.py"
        self.given(btoptions.OptionSwitch.BOLTFILE_LONG, boltfile)
        self.assertEqual(self.options.bolt_file, boltfile)

    def test_sets_boltfile_with_short_switch(self):
        boltfile = "a_bolt_file.py"
        self.given(btoptions.OptionSwitch.BOLTFILE_SHORT, boltfile)
        self.assertEqual(self.options.bolt_file, boltfile)

    def test_default_log_level_is_properly_initialized(self):
        self.assertEqual(self.default_options.log_level, btoptions.Default.LOG_LEVEL)

    def test_sets_log_level_with_long_switch(self):
        log_level = "error"
        self.given(btoptions.OptionSwitch.LOG_LEVEL_LONG, log_level)
        self.assertEqual(self.options.log_level, logging.ERROR)

    def test_sets_log_level_with_short_switch(self):
        log_level = "debug"
        self.given(btoptions.OptionSwitch.LOG_LEVEL_SHORT, log_level)
        self.assertEqual(self.options.log_level, logging.DEBUG)

    def test_converts_correctly_from_log_level_string_to_logging_level(self):
        # NOTSET
        self.verify_log_level("", logging.NOTSET)
        self.verify_log_level("n", logging.NOTSET)
        self.verify_log_level("notset", logging.NOTSET)
        # DEBUG
        self.verify_log_level("d", logging.DEBUG)
        self.verify_log_level("dbg", logging.DEBUG)
        self.verify_log_level("debug", logging.DEBUG)

    def test_default_log_file_is_properly_initialized(self):
        self.assertEqual(self.default_options.log_file, btoptions.Default.LOG_FILE)

    def test_sets_the_log_file_with_long_switch(self):
        log_file = "log.txt"
        self.given(btoptions.OptionSwitch.LOG_FILE_LONG, log_file)
        self.assertEqual(self.options.log_file, log_file)

    def test_sets_the_log_file_with_short_switch(self):
        log_file = "log.txt"
        self.given(btoptions.OptionSwitch.LOG_FILE_SHORT, log_file)
        self.assertEqual(self.options.log_file, log_file)

    def test_continue_on_error_is_properly_initialized(self):
        self.assertEqual(
            self.default_options.continue_on_error, btoptions.Default.CONTINUE_ON_ERROR
        )

    def test_sets_continue_on_error_with_long_switch(self):
        self.given(btoptions.OptionSwitch.CONTINUE_ON_ERROR_LONG)
        self.assertTrue(self.options.continue_on_error)

    def given(self, *args):
        self.options = btoptions.Options(args)

    def verify_log_level(self, str_level, expected):
        self.given(btoptions.OptionSwitch.LOG_LEVEL_LONG, str_level)
        self.assertEqual(self.options.log_level, expected)


if __name__ == "__main__":
    unittest.main()



import unittest

import bolt.errors as bt_errors


class TestInvalidTaskError(unittest.TestCase):
    # I need to check what InvalidTaskError is doing now, and evaluate if it
    # needs to provide more information.
    pass


class TestInvalidConfigurationError(unittest.TestCase):
    def setUp(self):
        super(TestInvalidConfigurationError, self).setUp()
        self.exc = bt_errors.InvalidConfigurationError()

    def test_returns_expected_string_representation(self):
        expected = "InvalidConfigurationError()"
        self.assertEqual(str(self.exc), expected)

    def test_representation_matches_string(self):
        self.assertEqual(repr(self.exc), str(self.exc))


class TestConfigurationParameterError(unittest.TestCase):
    def setUp(self):
        super(TestConfigurationParameterError, self).setUp()
        self.param_name = "parameter"
        self.exc = bt_errors.ConfigurationParameterError(self.param_name)

    def test_provides_parameter_name(self):
        self.assertEqual(self.exc.parameter, self.param_name)

    def test_returns_expected_string_representation(self):
        expected = "ConfigurationParameterError('parameter')"
        self.assertEqual(str(self.exc), expected)


class TestRequiredParameterMissingError(unittest.TestCase):
    def setUp(self):
        super(TestRequiredParameterMissingError, self).setUp()
        self.param_name = "required"
        self.exc = bt_errors.RequiredParameterMissingError(self.param_name)

    def test_provides_parameter_name(self):
        self.assertEqual(self.exc.parameter, self.param_name)

    def test_returns_expected_string_representation(self):
        expected = "RequiredParameterMissingError('required')"
        self.assertEqual(str(self.exc), expected)


class TestConfigurationValueError(unittest.TestCase):
    def setUp(self):
        super(TestConfigurationValueError, self).setUp()
        self.param_name = "parameter"
        self.value = "value"
        self.exc = bt_errors.ConfigurationValueError(self.param_name, self.value)

    def test_provides_parameter_name(self):
        self.assertEqual(self.exc.parameter, self.param_name)

    def test_provides_invalid_value(self):
        self.assertEqual(self.exc.value, self.value)

    def test_returns_expected_string_representation(self):
        expected = "ConfigurationValueError('parameter', 'value')"
        self.assertEqual(str(self.exc), expected)


class TestTaskError(unittest.TestCase):
    def setUp(self):
        super(TestTaskError, self).setUp()
        self.exc = bt_errors.TaskError()

    def test_returns_expected_string_representation(self):
        expected = "TaskError()"
        self.assertEqual(str(self.exc), expected)


if __name__ == "__main__":
    unittest.main()



"""
mkdir
-----

Creates the directory specified, including intermediate directories, if they
do not exist::

    config = {
        'mkdir': {
            'directory': 'several/intermediate/directories'
        }
    }
"""
import logging
import os

import bolt.api as api


class ExecuteMKDir(api.Task):
    def _configure(self):
        self.directory = self._require("directory")
        logging.debug(
            "Creating directory: {directory}".format(directory=self.directory)
        )

    def _execute(self):
        if not os.path.exists(self.directory):
            self._create_directories()

    def _create_directories(self):
        os.makedirs(self.directory)


def register_tasks(registry):
    registry.register_task("mkdir", ExecuteMKDir())



"""
coverage
--------

This task allows to measure code coverage of source code executed by the specified
task. The following is a sample configuration:

..  code-block:: python

    config = {
        'coverage': {
            'task': 'registered-task',
            'include': './directory/for/source',
            'output': 'directory/to/write/results'
        }
    }

The specified task must exercise the code that will be measured.
"""
import logging

import bolt
import bolt.api as api


class ExecuteCoverage(api.Task):
    def _configure(self):
        self.task_name = self._require("task")
        self.include_dir = self._require("include")
        self.out_dir = self._require("output")
        logging.info(
            "Code coverage for {task}. Output at {directory}".format(
                task=self.task_name, directory=self.out_dir
            )
        )

    def _execute(self):
        import coverage.control as cov

        controller = cov.Coverage(auto_data=False, branch=True, source=self.include_dir)
        controller.start()
        bolt.run_task(self.task_name)
        controller.stop()
        controller.html_report(directory=self.out_dir)


def register_tasks(registry):
    registry.register_task("coverage", ExecuteCoverage())



import unittest

import bolt.api as api
import bolt._btconfig as config
import bolt._btregistry as registry

import bolt._btrunner as runner


class TestTaskRunner(unittest.TestCase):
    def setUp(self):
        self.setUpRegistry()
        self.setUpConfig()
        self.subject = runner.TaskRunner(self.config_mgr, self.registry, False)
        self.executed_tasks = []
        return super(TestTaskRunner, self).setUp()

    def setUpRegistry(self):
        self.registry = registry.TaskRegistry()
        self.registry.register_task("task_1", self.task_1_callback)
        self.registry.register_task("task_2", self.task_2_callback)
        self.registry.register_task("invalid", self.invalid_task)
        self.registry.register_task("multi_task", ["task_1", "task_2"])
        self.registry.register_task("configurable", self.configurable_task)
        self.registry.register_task("failing_task", self.failing_task)
        self.registry.register_task(
            "failing_task_throug_return_code", self.failing_task_with_error_code
        )
        self.registry.register_task(
            "multi_task_with_failures", ["task_1", "failing_task", "task_2"]
        )
        self.tear_down_task = TearDownTask()
        self.registry.register_task("has_tear_down", self.tear_down_task)
        self.registry.register_task(
            "partially_executed", ["failing_task", "has_tear_down"]
        )

    def setUpConfig(self):
        self.config = {"configurable": {"param": "value"}}
        self.config_mgr = config.ConfigurationManager(self.config)

    def test_runs_the_specified_task(self):
        self.given("task_1")
        self.expect_executed("task_1")

    def test_configuration_is_passed_to_task_by_named_parameter(self):
        self.given("task_2")
        self.expect_executed("task_2")

    def test_raises_if_task_definition_does_not_meet_spec(self):
        with self.assertRaises(Exception):
            self.given("invalid")

    def test_executes_all_tasks_in_multi_task(self):
        self.given("multi_task")
        self.expect_executed("task_1")
        self.expect_executed("task_2")

    def test_task_is_executed_with_correct_configuration(self):
        self.given("configurable")
        self.expect_executed("configurable")

    def test_build_is_enforced_before_run(self):
        with self.assertRaises(Exception):
            subject = runner.TaskRunner(self.config_mgr, self.registry, False)
            subject.run()

    def test_build_fails_if_a_task_is_not_registered(self):
        with self.assertRaises(Exception):
            self.subject.build("inexistent")

    def test_exits_if_task_raises_exception(self):
        with self.assertRaises(api.TaskFailedError):
            self.given("failing_task")

    def test_exits_if_task_does_not_return_zero(self):
        with self.assertRaises(api.TaskFailedError):
            self.given("failing_task_throug_return_code")

    def test_continues_on_error_if_specified(self):
        continue_on_error = True
        self.subject = runner.TaskRunner(
            self.config_mgr, self.registry, continue_on_error
        )
        self.given("multi_task_with_failures")
        self.expect_executed("task_2")

    def test_calls_tear_down_on_tasks_that_implement_it(self):
        self.given("has_tear_down")
        self.assertTrue(self.tear_down_task.tear_down_called)

    def test_does_not_call_teardown_if_script_fails_before_executing_task(self):
        with self.assertRaises(api.TaskFailedError):
            self.given("partially_executed")
            self.assertFalse(self.tear_down_task.tear_down_called)

    def test_uses_same_execution_context_with_all_tasks(self):
        self.given("multi_task")
        self.assertTrue(self.context_shared)

    def given(self, task_name):
        try:
            self.subject.run(task_name)
        finally:
            self.subject.tear_down()

    def expect_executed(self, task_name):
        self.assertIn(task_name, self.executed_tasks)

    def task_1_callback(self, **kwargs):
        context = kwargs.get("context")
        context["shared-property"] = 1
        self.executed_tasks.append("task_1")

    def task_2_callback(self, **kwargs):
        context = kwargs.get("context")
        if context.get("shared-property") == 1:
            self.context_shared = True
        self.executed_tasks.append("task_2")

    def invalid_task(self, noconfig):
        pass

    def configurable_task(self, **kwargs):
        config = kwargs.get("config")
        param = config.get("param")
        self.assertEqual(param, "value")
        self.executed_tasks.append("configurable")

    def failing_task(self, **kwargs):
        raise api.TaskFailedError()

    def failing_task_with_error_code(self, **kwargs):
        return 1


class TearDownTask:
    def __init__(self):
        self.called = False
        self.tear_down_called = False
        return super(TearDownTask, self).__init__()

    def __call__(self, **kwargs):
        self.called = True

    def tear_down(self):
        self.tear_down_called = True


if __name__ == "__main__":
    unittest.main()



"""
conttest
--------

This task uses ``conttest`` to monitor a directory for changes and executes the specified
task everytime a change is made. The following configuration is supported::

    config = {
        'conttest': {
            'task': 'registered_task',
            'directory': './directory/to/monitor/'
        }
    }

The ``task`` parameter is the task to be executed and must be registered in ``boltfile.py``.
The ``directory`` parameter is the directory (including sub-directories) to monitor for
changes.

To use this task, you need to have ``conttest`` installed, which you can do by calling::

    pip install conttest
"""
import logging
import os

import bolt
import bolt.api as api


class ExecuteConttest(api.Task):
    def _configure(self):
        self.task_name = self._require("task")
        self.directory = self._optional("directory", os.getcwd())
        self.continue_on_error = True
        logging.info(
            'Executing continously "{task}" at {directory}'.format(
                task=self.task_name, directory=self.directory
            )
        )

    def _execute(self):
        import conttest.conttest as ct

        try:
            ct.watch_dir(self.directory, self._execute_assigned_task, method=ct.TIMES)
        except KeyboardInterrupt:
            logging.info("Exiting continuous execution")

    def _execute_assigned_task(self):
        bolt.run_task(self.task_name, self.continue_on_error)
        logging.info("Press <ctrl+c> to exit")


def register_tasks(registry):
    registry.register_task("conttest", ExecuteConttest())



from setuptools import setup, find_packages
from m2r2 import parse_from_file
from os import path
import bolt.about as about

readme_file = path.join(path.dirname(path.abspath(__file__)), 'README.md')
readme = parse_from_file(readme_file)

packages = find_packages()
entry_points = {
    'console_scripts': [
        'bolt = bolt:run'
    ]
}

setup(
    name=about.project,
    version=about.release,
    description=about.description,
    long_description=readme,
    author=about.author,
    author_email=about.author_email,
    url=about.url,
    license=about.license,
    keywords=about.keywords,
    classifiers=about.classifiers,
    packages=packages,
    entry_points=entry_points
)



"""
.. _task-set-vars:

set-vars
--------

Sets environment variables for all specified variable:value pairs. The following
shows how the task is configured:

..  code-block:: python

    config = {
        'set-vars': {
            'vars': {
                'STRING_VAR': 'string_value',
                'INT_VAR': 10
            }
        }
    }


Numeric vars will be converted to their integer representation.
"""
import os

import bolt.api as api


class SetVarsTask(api.Task):
    def _configure(self):
        self.vars = self._require("vars")

    def _execute(self):
        for var in self.vars:
            os.environ[var] = str(self.vars[var])


def register_tasks(registry):
    registry.register_task("set-vars", SetVarsTask())



"""
shell
-----

The ``shell`` task allows executing a shell command with specified arguments
inside the bolt execution context. This task comes handy when no bolt
specific implementation has been provided for a particular task or to invoke
an existing script that should be included as part of the process.

The trade-off of using this task is that commands are system specific and
it makes it harder to implement a cross-platform ``boltfile.py``.

The task takes a ``command`` parameter specifying the command to be executed,
and an ``arguments`` option that must be set to a list of string for each of
the command line argument tokens to be passed to the tool.

The following example shows how to invoke an existing |python|_ script that
takes a few parameters::

    config = {
        'shell': {
            'command': 'python',
            'arguments': ['existing_script.py', '--with-argument', '-f', '--arg-with', 'a_value']
        }
    }

..  todo::  Find a better example.
"""
import logging
import subprocess as sp

import bolt.api as api


class ShellExecuteTask(api.Task):
    def _configure(self):
        self.command = self._require("command")
        self.command_line = [self.command]
        arguments = self._optional("arguments", [])
        self.command_line.extend(arguments)

    def _execute(self):
        logging.debug("Shell command line: %s", repr(self.command_line))
        result = sp.call(self.command_line)
        if result != 0:
            raise ShellError(result)


def register_tasks(registry):
    registry.register_task("shell", ShellExecuteTask())


class ShellError(api.TaskFailedError):
    def __init__(self, shell_code):
        super(ShellError, self).__init__(shell_code)

    def __repr__(self):
        return "ShellError({code})".format(code=self.code)



"""
"""
import logging

import bolt.api as api


class TaskRegistry:
    """ """

    def __init__(self):
        self._tasks = {}

    def register_task(self, name, task):
        """

        :param name:
        :param task:
        :return:
        """
        if not self._is_valid_task(task):
            raise InvalidTaskTypeError(type(task))
        self._tasks[name] = task
        msg = "{task_name} task is registered.".format(task_name=name)
        logging.debug(msg)

    def register_module_tasks(self, module):
        """ """
        module.register_tasks(self)

    def get(self, name):
        """

        :param name:
        :return:
        """
        task_name = self._extract_task_name(name)
        return self._tasks[task_name]

    def _is_valid_task(self, task):
        is_list = isinstance(task, list)
        if is_list:
            for t in task:
                if not isinstance(t, str):
                    return False
        return callable(task) or is_list

    def _extract_task_name(self, full_task):
        return full_task.split(".")[0]


class InvalidTaskTypeError(api.BoltError):
    """ """

    def __init__(self, task_type):
        super(InvalidTaskTypeError, self).__init__(100)
        self.task_type = task_type

    def __repr__(self):
        return "InvalidTaskTypeError({t})".format(t=self.task_type)



import unittest

import bolt._btconfig as config


class TestConfigurationManager(unittest.TestCase):
    def setUp(self):
        self.configuration = {
            "task": {
                "option": 1,
                "overridable": "initial",
                "child": {
                    "overridable": "inchild",
                    "grandchild": {"overridable": "ingrandchild"},
                },
            }
        }
        self.subject = config.ConfigurationManager(self.configuration)

    def test_returns_empty_configuration_if_task_id_does_not_exists(self):
        config = self.subject.get("inexistent")
        self.assertFalse(config)

    def test_returns_the_specified_configuration(self):
        self.given("task")
        self.expect("option", 1)

    def test_options_can_be_overriden_in_child_configurations(self):
        self.given("task.child")
        self.expect("overridable", "inchild")

    def test_child_configuration_maintains_parent_values_if_not_overriden(self):
        self.given("task.child")
        self.expect("option", 1)

    def test_supports_many_levels_of_child_indentation(self):
        self.given("task.child.grandchild")
        self.expect("overridable", "ingrandchild")

    def given(self, key):
        self.result = self.subject.get(key)

    def expect(self, option, expected):
        actual_value = self.result.get(option)
        self.assertEqual(actual_value, expected)


if __name__ == "__main__":
    unittest.main()
